# -*- coding: utf-8 -*-

import os
import globalPluginHandler
import speechDictHandler
import ui
import wx
import gui
# import addonHandler
from logHandler import log
from configobj import ConfigObj
from validate import Validator
from cStringIO import StringIO

dicPath = os.path.join(os.path.dirname(__file__), "dics")
dicList = os.listdir(dicPath)

config_FileName = os.path.join(os.path.dirname(__file__), "dictionaryManager.ini")
confspec = ConfigObj(StringIO("""#Configuration file

[Dictionaries settings]
	dics = list(default=list())
"""))
confspec.newlines = "\r\n"
conf = ConfigObj(config_FileName, configspec = confspec, indent_type = "\t", encoding="UTF-8")

val = Validator()
conf.validate(val)
addedDics = conf["Dictionaries settings"]["dics"]

class Dictionary(object):

	def __init__(self, dicName, checkStatusList):
		super(Dictionary, self).__init__()
		self.name = dicName
		if self.name in checkStatusList:
			# Translators: announced when the user selects one of the dictionaries.
			self.status = _("Added")
		else:
			# Translators: announced when the user unselects one of the dictionaries.
			self.status = _("Not added")

class DictionariesDialog(gui.SettingsDialog):

		# Translators: The title of the dictionaries dialog.
	title = _("Dictionary Manager: %d") % len(dicList)

	def makeSettings(self, settingsSizer):
		entriesSizer=wx.BoxSizer(wx.VERTICAL)
		# Translators: the label for the dictionaries list for Dictionary Manager.
		entriesLabel=wx.StaticText(self, -1, label=_("&Add the dictionaries you want to use for Dictionary Manager. Press Space to toggle between added or not added."))
		entriesSizer.Add(entriesLabel)
		self.dicList=wx.ListCtrl(self,-1,style=wx.LC_REPORT|wx.LC_SINGLE_SEL,size=(550, 350))
		# Translators: The label for a column in dictionaries list used to identify a dictionary name.
		self.dicList.InsertColumn(0,_("Dictionary"), width=300)
		# Translators: The label for a column in dictionaries list used to identify dictionary status.
		self.dicList.InsertColumn(2,_("Status"), width=150)
		self.dicList.Bind(wx.EVT_LIST_ITEM_FOCUSED, self.onListItemSelected)
		self.dicList.Bind(wx.EVT_CHAR, self.onListChar)
		entriesSizer.Add(self.dicList, proportion=8)
		entryButtonsSizer=wx.BoxSizer(wx.HORIZONTAL)
		# Translators: The label for a button.
		self.downButton=wx.Button(self,label=_("&Down dictionary"))
		self.downButton.Disable()
		self.downButton.Bind(wx.EVT_BUTTON,self.onDown)
		entryButtonsSizer.Add(self.downButton)
		# Translators: The label for a button.
		self.upButton=wx.Button(self,label=_("&Up dictionary"))
		self.upButton.Disable()
		self.upButton.Bind(wx.EVT_BUTTON,self.onUp)
		entryButtonsSizer.Add(self.upButton)
		settingsSizer.Add(entryButtonsSizer)
		self.refreshDicList(0)

	def postInit(self):
		self.dicList.SetFocus()

	def OnToggleClick(self, evt):
		try:
			dicName = dicList[self.index]
		except:
			return
		global addedDics
		if dicName not in addedDics:
			addedDics.append(dicName)
		else:
			addedDics.remove(dicName)
		self.refreshDicList(self.index)

	def onListItemSelected(self, evt):
		self.index = evt.GetIndex()
		if self.index < len(dicList)-1:
			self.downButton.Enable()
		else:
			self.downButton.Enable(False)
		if self.index > 0:
			self.upButton.Enable()
		else:
			self.upButton.Enable(False)

	def saveDefaultDics(self):
		conf["Dictionaries settings"]["dics"] = addedDics
		try:
			conf.validate(val, copy=True)
			conf.write()
			log.info("dictionaryManager add-on configuration saved")
		except Exception as e:
			log.warning("Could not save dictionaryManager add-on configuration")
			log.debugWarning("", exc_info=True)
			#raise e

	def onOk(self, evt):
		super(DictionariesDialog, self).onOk(evt)
		if len(addedDics) > 0:
			self.saveDefaultDics()

	def refreshDicList(self, activeIndex):
		self.dicList.DeleteAllItems()
		for dicName in dicList:
			dic = Dictionary(dicName, addedDics)
			self.dicList.Append((dic.name, dic.status))
		self.dicList.Select(activeIndex,on=1)
		self.dicList.SetItemState(activeIndex,wx.LIST_STATE_FOCUSED,wx.LIST_STATE_FOCUSED)

	def onListChar(self, evt):
		if evt.KeyCode == wx.WXK_RETURN:
			# Activate the OK button.
			self.ProcessEvent(wx.CommandEvent(wx.wxEVT_COMMAND_BUTTON_CLICKED, wx.ID_OK))
		elif evt.KeyCode == wx.WXK_SPACE:
			self.OnToggleClick(None)
		else:
			evt.Skip()

	def onDown(self,evt):
		global dicList
		dicList[self.index], dicList[self.index+1] = dicList[self.index+1], dicList[self.index]
		self.refreshDicList(self.index+1)

	def onUp(self,evt):
		global dicList
		dicList[self.index], dicList[self.index-1] = dicList[self.index-1], dicList[self.index]
		self.refreshDicList(self.index-1)

class GlobalPlugin(globalPluginHandler.GlobalPlugin):

	def __init__(self):
		super(globalPluginHandler.GlobalPlugin, self).__init__()
		self._sD = speechDictHandler.SpeechDict()
		for dic in dicList:
			self._sD.load(os.path.join(dicPath, dic))
			speechDictHandler.dictionaries["temp"].extend(self._sD)
		# Gui
		self.menu = gui.mainFrame.sysTrayIcon.preferencesMenu
		self.addonMenu = wx.Menu()
		self.mainItem = self.menu.AppendSubMenu(self.addonMenu,
		# Translators: the name of addon submenu.
		_("Dictionary Manager"),
		# Translators: the tooltip text for addon submenu.
		_("Dictionary Manager menu"))
		self.insertItem = self.addonMenu.Append(wx.ID_ANY,
		# Translators: the name for an item of addon submenu.
		_("&List of dictionaries..."),
		# Translators: the tooltip text for an item of addon submenu.
		_("Shows a dialog to manage a list of dictionaries"))
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onDictionariesDialog, self.insertItem)

	def terminate(self):
		try:
			self.menu.RemoveItem(self.mainItem)
		except wx.PyDeadObjectError:
			pass

	def onDictionariesDialog(self, evt):
		gui.mainFrame._popupSettingsDialog(DictionariesDialog)
